{% extends "base.html" %}
{% block header %}
<h1>{{post['title']}}</h1>
{% endblock header %}
{% block content %}
<h2>Posted by {{post['author']}} on
    <i>{{post['month']}} {{post['day']}}, {{post['year']}}</i>
</h2>
{%if post.get('original')%}
  <h3>This was originally posted at <a href="{{post['original']}}">{{post['original']}}</a>.</h3>
{%endif%}
<br/>
{{post['post']}}
{% endblock content %}
{% block footer %}
{% endblock footer %}
