<ul>{%for job in jobs%}
  {%if not job['open']%}
  <li><a href="{{root}}/jobs/{{job['_id']}}.html"><b>{{job['title']}}</b>,
    opened on <i>{{job['month']}} {{job['day']}}, {{job['year']}}</i>.
    (#{{job['_id']}})</a>
  </li>
{%endif%}
{%endfor%}</ul>
