<!DOCTYPE HTML>
<!-- Strongly Typed by HTML5 UP html5up.net | @n33co -->
<html>
<head>
  <title>{{title}}</title>
  <meta charset="utf-8" />
  <meta name="viewport" content="width=device-width, initial-scale=1" />
  <!--[if lte IE 8]><script src="{{static}}/js/ie/html5shiv.js"></script><![endif]-->
  <link rel="stylesheet" href="{{static}}/css/main.css" />
  <!--[if lte IE 8]><link rel="stylesheet" href="{{static}}/css/ie8.css" /><![endif]-->
</head>
<body class="no-sidebar">
  <div id="page-wrapper">

    <!-- Header -->
    <div id="header-wrapper">
      <div id="header" class="container">
        {% block header %}
        {% endblock header %}

        <!-- Nav -->
        <nav id="nav">
          <ul style="list-style:none;">
            <li><a class="icon fa-home" href="{{root}}/index.html"><span>About</span></a></li>
            <li>
              <a class="icon fa-users" href="{{root}}/people"><span>People</span></a>
              <ul style="list-style:none;">{% for p in people if p.get('active', True) %}
                <li><a href="{{root}}/people/{{p['_id']}}.html">{{p['name']}}</a></li>
              {% endfor %}
                <li><a href="{{root}}/former">Former Members</a></li>
              </ul>
            </li>
            <li><a class="icon fa-cog" href="{{root}}/projects.html"><span>Projects</span></a></li>
            <li><a class="icon fa-pencil" href="{{root}}/blog"><span>Blog</span></a></li>
            <li><a class="icon fa-user-plus" href="{{root}}/jobs"><span>Jobs</span></a></li>
          </ul>
        </nav>

      </div>
    </div> <!-- Header -->

    <!-- Main -->
    <div id="main-wrapper">
      <div id="main" class="container">
        <div id="content">
          {% block content %}
          {% endblock content %}
        </div>
      </div>
    </div> <!-- Main -->

    <!-- Footer -->
    <div id="footer-wrapper">
      <div id="footer" class="container">
          {% block footer %}
          {% endblock footer %}
      </div>
      <div id="copyright" class="container">
        <ul class="links">
          <li><img src="{{static}}/logo-curves.svg" height="100px" style="vertical-align:bottom;" /></li>
          <li>&copy; Anthony Scopatz. All rights reserved.</li>
          <li>Design: <a href="http://html5up.net">HTML5 UP</a></li>
        </ul>
      </div>
    </div> <!-- Footer -->

  </div> <!-- page-wrapper -->

  <!-- Scripts -->
  <script src="{{static}}/js/jquery.min.js"></script>
  <script src="{{static}}/js/jquery.dropotron.min.js"></script>
  <script src="{{static}}/js/skel.min.js"></script>
  <script src="{{static}}/js/skel-viewport.min.js"></script>
  <script src="{{static}}/js/util.js"></script>
  <!--[if lte IE 8]><script src="{{static}}/js/ie/respond.min.js"></script><![endif]-->
  <script src="{{static}}/js/main.js"></script>
</body>
</html>
